/* opensock.c */
/*  $Id$ */

#include "opensocks.h"
#define MAXSIZE 32768

int WindowsSocketStartup()
{
	WSADATA wsaData;
	int err;
	unsigned int wVersionRequested;

	wVersionRequested = MAKEWORD( 1, 1 );

	err = WSAStartup( wVersionRequested, &wsaData );   /* fill addr struct */
	
	return err;
}

int CreateReceiveSocket(struct sockaddr_in *addr, 
			int port, char * group)
{

	int err;
	int len_addr, one=1;
	struct hostent *hp;
	int sock;
	struct ip_mreq imr;

	WindowsSocketStartup();

	len_addr = sizeof(*addr);
	memset((char *)addr, 0, len_addr); /* reset addr struct to 0 */
	addr->sin_family=AF_INET;
	addr->sin_port = htons((short)port);
   
	if(((hp=gethostbyname(group))!=NULL) && (hp->h_addrtype== AF_INET))
	{
		memcpy((char *)&addr->sin_addr, hp->h_addr, hp->h_length);
	}
	else
	{
		addr->sin_addr.s_addr = inet_addr(group);
		if(addr->sin_addr.s_addr == -1)
		{
			fprintf(stderr, 
				"Create receive socket: invalid group %s\n", group);
			exit(-1);
		}
	}

	if( (sock = socket(AF_INET,  SOCK_DGRAM, 0))<0)
	{
		perror("cannot create socket");
		exit(-1);
	}
   
   if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
		 (char *)&one, sizeof(one))==-1)
     {
	perror("setsockupt 'SO_REUSEADDR'");
     }

   addr->sin_addr.s_addr = INADDR_ANY;

	err = bind(sock, (struct sockaddr *)addr, len_addr);
	if(err)
	{
		perror("bind");
		err = WSAGetLastError();
		fprintf(stderr,"cannot bind socket to address %s port %d\n", 
			inet_ntoa(addr->sin_addr), port);
		exit(1);
	}

//   optval = TRUE;
//   setsockopt(sock, SOL_SOCKET, SO_BROADCAST, 
//	   (char *) &optval, sizeof(BOOL));
//   {
//	   err = WSAGetLastError();
//   }

//   optval = FALSE;
//   if(setsockopt(sock, IPPROTO_IP, IP_MULTICAST_LOOP, 
//	   (char *)&optval, sizeof(int)))
//   {
//	   err = WSAGetLastError();
//   }

//   imr.imr_multiaddr.s_addr = addr->sin_addr.s_addr;
   imr.imr_multiaddr.s_addr = inet_addr(group);
   imr.imr_interface.s_addr = INADDR_ANY;
    
//   optlen=4;
//   optval = 50000;
//   if(setsockopt(sock,SOL_SOCKET, SO_RCVBUF, (char *)&optval, optlen)!=0)
//     {
//	perror("setsockopt failed");
//	getsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&optval, &optlen);
//	fprintf(stderr,"SO_RCVBUF = %d\n", optval);
//     }
  
   
   
   if(setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *) &imr,
                 sizeof(struct ip_mreq)) == -1)
   {
	   fprintf(stderr,"cannot join group %s \n", group);
	   perror("setsockopt()");
	   err = WSAGetLastError();
	   if(errno==WSAEADDRNOTAVAIL)
	   {
		   fprintf(stderr,"Try: /usr/bin/route add 224.0.0.0 192.1.1.1.1 0 \n");
	   }
	   else
	   {
		   fprintf(stderr, 
			   "Does your kernel support ip multicast extensions ?\n");
	   }
	   exit(-1);
   }

   return sock;
}

/* CreateSendSocket */
int CreateSendSocket(struct sockaddr_in *addr, int *len_addr, int port, char *group, int ttl)
{
   struct sockaddr_in addr_local;
   int sock;
   u_int iport;
   struct sockaddr_in my_addr;
   struct hostent *hp;
   int len_my_addr;
   int one=1;
   struct ip_mreq imr;
   char Hostname[100];
   unsigned char cttl=ttl;
   
   WindowsSocketStartup();
   iport = port;
   gethostname(Hostname, 100);
   
   if((sock = socket(AF_INET, SOCK_DGRAM, 0))<0)
     {
	perror("cannot create datagram socket");
	exit(1);
     }
   memset((char *)&addr_local, 0, sizeof(addr_local));
   addr_local.sin_family = AF_INET;
   addr_local.sin_addr.s_addr = htonl(INADDR_ANY);
   addr_local.sin_port = htons(0);
   if(bind(sock, (struct sockaddr *) &addr_local, sizeof(addr_local)) < 0)
     {
	perror("bind");
	fprintf(stderr, "Cannot bind socket to address %s port 0\n", 
		inet_ntoa(addr_local.sin_addr));
	exit(1);
     }
   else
     {
	struct sockaddr name; 
	int namelen=sizeof(name);
	
	getsockname(sock, &name, &namelen);
	memcpy((char *)&addr_local,(char *)&name, namelen);
     }
   *len_addr = sizeof(*addr);
   memset((char *)addr, 0, *len_addr);
   addr->sin_port = htons(iport);
   addr->sin_addr.s_addr = inet_addr(group);
   addr->sin_family = AF_INET;
   
   setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, (unsigned char *)&cttl, sizeof(cttl));
   
   len_my_addr = sizeof(my_addr);
   memset ((char *) &my_addr, 0, len_my_addr);
   
   if(((hp=gethostbyname(Hostname))!=NULL) &&
      (hp->h_addrtype == AF_INET))
     {
	memcpy((char *)&my_addr.sin_addr, hp->h_addr, hp->h_length);
     }
   else
     {
	my_addr.sin_addr.s_addr = Hostname;
	printf("my_addr.sin_addr.s_addr %x", my_addr.sin_addr.s_addr);
	if(my_addr.sin_addr.s_addr == -1)
	  {
	     fprintf(stderr, "CreateSendSocket: Invalid interface %s \n", Hostname);
	     exit(1);
	  }
     }
   imr.imr_multiaddr.s_addr = inet_addr(group);
   imr.imr_interface.s_addr = INADDR_ANY;
   
   if(setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
		 (char *)&imr, sizeof(struct ip_mreq))==-1)
     {
	fprintf(stderr,"Cannot join group %s\n", group);
	perror("setsockopt 'IP_ADD_MEMBERSHIP'");
	if(errno==WSAEADDRNOTAVAIL)
	  {
	     fprintf(stderr,"Try: /sbin/route add -net 224.0.0.0 eth0\n");
	  }
	exit(1);
     }
   if(setsockopt(sock,SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one))==-1)
     {
	perror("setsockopt 'SO_REUSEADDR'");
     }
   return sock;
}

/*
 * 
 * Cleanup Socket function
 *
 */
int cleanup(int sock1)
{
   int retval;

   retval = closesocket(sock1);
   if(retval)
     perror("close");
   exit(0);

   return 0;
}
